<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) 2016 Amasty (https://www.amasty.com)
 * @package Amasty_Xlanding
 */

namespace Amasty\Xlanding\Model\Rule\Condition;

class IsNewByPeriod extends AbstractCondition
{
    protected $_inputType = 'select';
    protected $_string;
    protected $_storeManager;
    protected $_dateTime;
    protected $_date;

    public function __construct(
        \Magento\Rule\Model\Condition\Context $context,
        \Magento\Backend\Helper\Data $backendData,
        \Magento\Eav\Model\Config $config,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\Catalog\Api\ProductRepositoryInterface $productRepository,
        \Magento\Catalog\Model\ResourceModel\Product $productResource,
        \Magento\Eav\Model\ResourceModel\Entity\Attribute\Set\Collection $attrSetCollection,
        \Magento\Framework\Locale\FormatInterface $localeFormat,
        \Magento\Framework\Stdlib\StringUtils $string,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\Stdlib\DateTime\DateTime $date,
        \Magento\Framework\Stdlib\DateTime $dateTime,
        array $data = []
    ){
        $this->_string = $string;
        $this->_storeManager = $storeManager;

        $this->_dateTime = $dateTime;
        $this->_date = $date;

        parent::__construct(
            $context,
            $backendData,
            $config,
            $productFactory,
            $productRepository,
            $productResource,
            $attrSetCollection,
            $localeFormat,
            $data
        );
    }



    public function getAttributeElementHtml()
    {
        return __('Is New');
    }

    public function getInputType()
    {
        return 'select';
    }

    public function getValueElementType()
    {
        return 'select';
    }

    protected function _getAttributeCode()
    {
        return 'is_stock';
    }

    protected function _prepareValueOptions()
    {
        $selectReady = $this->getData('value_select_options');
        $hashedReady = $this->getData('value_option');

        $selectOptions = [
            ['value' => 1, 'label' => 'Yes'],
            ['value' => 0, 'label' => 'No']
        ];

        $this->_setSelectOptions($selectOptions, $selectReady, $hashedReady);

        return $this;
    }

    public function collectValidatedAttributes($select)
    {
        $connection = $this->_productResource->getConnection();

        $alias = $this->_getAlias();

        $aliasFrom = $alias . '_from';
        $aliasTo = $alias . '_to';

        $attributeFrom = $this->_config->getAttribute(\Magento\Catalog\Model\Product::ENTITY, 'news_from_date');

        $attributeTo = $this->_config->getAttribute(\Magento\Catalog\Model\Product::ENTITY, 'news_to_date');

        $value = $this->_dateTime->formatDate($this->_date->gmtTimestamp());

        $conditionFrom = 'false';
        $conditionTo = 'false';

        if ($this->getValue() && $this->getOperatorForValidate() == '=='){
            $conditionFrom = $this->getOperatorCondition($aliasFrom . '.value', '<=', $value);
            $conditionTo = $this->getOperatorCondition($aliasTo . '.value', '>=', $value) . ' and ' . $aliasTo . '.value is null';
        } else {
            $conditionFrom = $this->getOperatorCondition($aliasFrom . '.value', '>', $value) . ' or ' . $aliasFrom . '.value is null';

            $conditionTo = $this->getOperatorCondition($aliasTo . '.value', '<', $value) . ' or ' . $aliasTo . '.value is null';
        }

//        $value     = $this->getValue();
//        $operator  = $this->getOperatorForValidate();


        $this->_condition = '(' . $conditionFrom . ') and (' . $conditionTo . ')';

        $mapTpl = 'search_index.entity_id = %1$s.entity_id AND %1$s.attribute_id = %2$d AND %1$s.store_id = %3$d and %4$s';

        $select->joinLeft(
            [
                $aliasFrom => $this->_productResource->getTable('catalog_product_entity_datetime')
            ],
            sprintf(
                $mapTpl,
                $aliasFrom,
                $attributeFrom->getId(),
                0,
                $conditionFrom
            ),
            []
        );

        $select->joinLeft(
            [
                $aliasTo => $this->_productResource->getTable('catalog_product_entity_datetime')
            ],
            sprintf(
                $mapTpl,
                $aliasTo,
                $attributeTo->getId(),
                0,
                $conditionTo
            ),
            []
        );
    }
}
